@@ -29,12 +29,13 @@
 #include <gst/audio/multichannel.h>
 
 #include <string.h>
+#include <math.h>
 
 GST_DEBUG_CATEGORY_EXTERN (riff_debug);
 #define GST_CAT_DEFAULT riff_debug
 
 /**
- * gst_riff_create_video_caps_with_data:
+ * gst_riff_create_video_caps:
  * @codec_fcc: fourCC codec for this codec.
  * @strh: pointer to the strh stream header structure.
  * @strf: pointer to the strf stream header structure, including any
@@ -74,6 +75,7 @@ gst_riff_create_video_caps (guint32 codec_fcc,
               "blue_mask", G_TYPE_INT, 0xff0000, NULL);
         } else {
           GST_WARNING ("Unhandled DIB RGB depth: %d", bpp);
+          return NULL;
         }
       } else {
         /* for template */
@@ -119,6 +121,7 @@ gst_riff_create_video_caps (guint32 codec_fcc,
       break;
 
     case GST_MAKE_FOURCC ('J', 'P', 'E', 'G'): /* generic (mostly RGB) MJPEG */
+    case GST_MAKE_FOURCC ('j', 'p', 'e', 'g'): /* generic (mostly RGB) MJPEG */
       caps = gst_caps_new_simple ("image/jpeg", NULL);
       if (codec_name)
         *codec_name = g_strdup ("JPEG Still Image");
@@ -229,6 +232,8 @@ gst_riff_create_video_caps (guint32 codec_fcc,
 
     case GST_MAKE_FOURCC ('H', '2', '6', '4'):
     case GST_MAKE_FOURCC ('h', '2', '6', '4'):
+    case GST_MAKE_FOURCC ('a', 'v', 'c', '1'):
+    case GST_MAKE_FOURCC ('A', 'V', 'C', '1'):
       caps = gst_caps_new_simple ("video/x-h264",
           "variant", G_TYPE_STRING, "itu", NULL);
       if (codec_name)
@@ -242,6 +247,14 @@ gst_riff_create_video_caps (guint32 codec_fcc,
         *codec_name = g_strdup ("VideoSoft H.264");
       break;
 
+    case GST_MAKE_FOURCC ('L', '2', '6', '4'):
+      /* http://www.leadcodecs.com/Codecs/LEAD-H264.htm */
+      caps = gst_caps_new_simple ("video/x-h264",
+          "variant", G_TYPE_STRING, "lead", NULL);
+      if (codec_name)
+        *codec_name = g_strdup ("Lead H.264");
+      break;
+
     case GST_MAKE_FOURCC ('S', 'E', 'D', 'G'):
       caps = gst_caps_new_simple ("video/mpeg",
           "mpegversion", G_TYPE_INT, 4, NULL);
@@ -300,7 +313,6 @@ gst_riff_create_video_caps (guint32 codec_fcc,
       break;
 
     case GST_MAKE_FOURCC ('M', 'P', 'G', '4'):
-    case GST_MAKE_FOURCC ('M', 'P', '4', 'S'):
       caps = gst_caps_new_simple ("video/x-msmpeg",
           "msmpegversion", G_TYPE_INT, 41, NULL);
       if (codec_name)
@@ -323,6 +335,7 @@ gst_riff_create_video_caps (guint32 codec_fcc,
         *codec_name = g_strdup ("Microsoft MPEG-4 4.3");
       break;
 
+    case GST_MAKE_FOURCC ('M', 'P', '4', 'S'):
     case GST_MAKE_FOURCC ('M', '4', 'S', '2'):
       caps = gst_caps_new_simple ("video/mpeg",
           "mpegversion", G_TYPE_INT, 4, NULL);
@@ -340,9 +353,11 @@ gst_riff_create_video_caps (guint32 codec_fcc,
 
     case GST_MAKE_FOURCC ('3', 'i', 'v', 'd'):
     case GST_MAKE_FOURCC ('3', 'I', 'V', 'D'):
+      caps = gst_caps_new_simple ("video/x-msmpeg",
+          "msmpegversion", G_TYPE_INT, 43, NULL);
       if (codec_name)
         *codec_name = g_strdup ("Microsoft MPEG-4 4.3");        /* FIXME? */
-      return gst_caps_from_string ("video/x-msmpeg, msmpegversion = (int) 43");
+      break;
 
     case GST_MAKE_FOURCC ('3', 'I', 'V', '1'):
     case GST_MAKE_FOURCC ('3', 'I', 'V', '2'):
@@ -355,11 +370,21 @@ gst_riff_create_video_caps (guint32 codec_fcc,
     case GST_MAKE_FOURCC ('d', 'v', 's', 'd'):
     case GST_MAKE_FOURCC ('C', 'D', 'V', 'C'):
       caps = gst_caps_new_simple ("video/x-dv",
-          "systemstream", G_TYPE_BOOLEAN, FALSE, NULL);
+          "systemstream", G_TYPE_BOOLEAN, FALSE,
+          "dvversion", G_TYPE_INT, 25, NULL);
       if (codec_name)
         *codec_name = g_strdup ("Generic DV");
       break;
 
+    case GST_MAKE_FOURCC ('D', 'V', '5', '0'):
+    case GST_MAKE_FOURCC ('d', 'v', '5', '0'):
+      caps = gst_caps_new_simple ("video/x-dv",
+          "systemstream", G_TYPE_BOOLEAN, FALSE,
+          "dvversion", G_TYPE_INT, 50, NULL);
+      if (codec_name)
+        *codec_name = g_strdup ("DVCPro50 Video");
+      break;
+
     case GST_MAKE_FOURCC ('W', 'M', 'V', '1'):
       caps = gst_caps_new_simple ("video/x-wmv",
           "wmvversion", G_TYPE_INT, 1, NULL);
@@ -389,6 +414,14 @@ gst_riff_create_video_caps (guint32 codec_fcc,
         *codec_name = g_strdup ("Microsoft Windows Media Advanced Profile");
       break;
 
+    case GST_MAKE_FOURCC ('W', 'V', 'C', '1'):
+      caps = gst_caps_new_simple ("video/x-wmv",
+          "wmvversion", G_TYPE_INT, 3, "fourcc", GST_TYPE_FOURCC,
+          codec_fcc, NULL);
+      if (codec_name)
+        *codec_name = g_strdup ("Microsoft Windows Media VC-1");
+      break;
+
     case GST_MAKE_FOURCC ('c', 'v', 'i', 'd'):
       caps = gst_caps_new_simple ("video/x-cinepak", NULL);
       if (codec_name)
@@ -706,7 +739,10 @@ gst_riff_wavext_add_channel_layout (GstCaps * caps, guint32 layout)
   if (!gst_structure_get_int (s, "channels", &num_channels))
     g_return_val_if_reached (FALSE);
 
-  if (num_channels < 2 || num_channels > MAX_CHANNEL_POSITIONS) {
+  /* In theory this should be done for 1 and 2 channels too but
+   * apparently breaks too many things currently.
+   */
+  if (num_channels <= 2 || num_channels > MAX_CHANNEL_POSITIONS) {
     GST_DEBUG ("invalid number of channels: %d", num_channels);
     return FALSE;
   }
@@ -740,6 +776,93 @@ gst_riff_wavext_add_channel_layout (GstCaps * caps, guint32 layout)
   return TRUE;
 }
 
+static gboolean
+gst_riff_wave_add_default_channel_layout (GstCaps * caps)
+{
+  GstAudioChannelPosition pos[8] = { GST_AUDIO_CHANNEL_POSITION_NONE, };
+  GstStructure *s;
+  gint nchannels;
+
+  s = gst_caps_get_structure (caps, 0);
+
+  if (!gst_structure_get_int (s, "channels", &nchannels))
+    g_return_val_if_reached (FALSE);
+
+  if (nchannels > 8) {
+    GST_DEBUG ("invalid number of channels: %d", nchannels);
+    return FALSE;
+  }
+
+  /* This uses the default channel mapping from ALSA which
+   * is used in quite a few surround test files and seems to be
+   * the defacto standard. The channel mapping from
+   * WAVE_FORMAT_EXTENSIBLE doesn't seem to be used in normal
+   * wav files like chan-id.wav.
+   * http://bugzilla.gnome.org/show_bug.cgi?id=489010
+   */
+  switch (nchannels) {
+    case 1:
+      pos[0] = GST_AUDIO_CHANNEL_POSITION_FRONT_MONO;
+      break;
+    case 8:
+      pos[7] = GST_AUDIO_CHANNEL_POSITION_SIDE_RIGHT;
+      pos[6] = GST_AUDIO_CHANNEL_POSITION_SIDE_LEFT;
+      /* fall through */
+    case 6:
+      pos[5] = GST_AUDIO_CHANNEL_POSITION_LFE;
+      /* fall through */
+    case 5:
+      pos[4] = GST_AUDIO_CHANNEL_POSITION_FRONT_CENTER;
+      /* fall through */
+    case 4:
+      pos[3] = GST_AUDIO_CHANNEL_POSITION_REAR_RIGHT;
+      pos[2] = GST_AUDIO_CHANNEL_POSITION_REAR_LEFT;
+      /* fall through */
+    case 2:
+      pos[1] = GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT;
+      pos[0] = GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT;
+      break;
+    default:
+      return FALSE;
+  }
+
+  gst_audio_set_channel_positions (s, pos);
+  return TRUE;
+}
+
+static guint32
+gst_riff_wavext_get_default_channel_mask (guint nchannels)
+{
+  guint32 channel_mask = 0;
+
+  /* Set the default channel mask for the given number of channels.
+   * http://www.microsoft.com/whdc/device/audio/multichaud.mspx
+   */
+  switch (nchannels) {
+    case 11:
+      channel_mask |= 0x00400;
+      channel_mask |= 0x00200;
+    case 9:
+      channel_mask |= 0x00100;
+    case 8:
+      channel_mask |= 0x00080;
+      channel_mask |= 0x00040;
+    case 6:
+      channel_mask |= 0x00020;
+      channel_mask |= 0x00010;
+    case 4:
+      channel_mask |= 0x00008;
+    case 3:
+      channel_mask |= 0x00004;
+    case 2:
+      channel_mask |= 0x00002;
+      channel_mask |= 0x00001;
+      break;
+  }
+
+  return channel_mask;
+}
+
 GstCaps *
 gst_riff_create_audio_caps (guint16 codec_id,
     gst_riff_strh * strh, gst_riff_strf_auds * strf,
@@ -752,52 +875,56 @@ gst_riff_create_audio_caps (guint16 codec_id,
 
   switch (codec_id) {
     case GST_RIFF_WAVE_FORMAT_PCM:     /* PCM */
+      channels_max = 8;
+
       if (strf != NULL) {
         gint ba = strf->blockalign;
         gint ch = strf->channels;
-        gint ws = strf->size;
+        gint wd, ws;
+
+        /* If we have an empty blockalign, we take the width contained in 
+         * strf->size */
+        if (ba != 0)
+          wd = ba * 8 / ch;
+        else
+          wd = strf->size;
+
+        if (strf->size > 32) {
+          GST_WARNING ("invalid depth (%d) of pcm audio, overwriting.",
+              strf->size);
+          strf->size = 8 * ((wd + 7) / 8);
+        }
+
+        /* in riff, the depth is stored in the size field but it just means that
+         * the _least_ significant bits are cleared. We can therefore just play
+         * the sample as if it had a depth == width */
+        /* For reference, the actual depth is in strf->size */
+        ws = wd;
 
         caps = gst_caps_new_simple ("audio/x-raw-int",
             "endianness", G_TYPE_INT, G_LITTLE_ENDIAN,
             "channels", G_TYPE_INT, ch,
-            "width", G_TYPE_INT, (int) (ba * 8 / ch),
-            "depth", G_TYPE_INT, ws, "signed", G_TYPE_BOOLEAN, ws != 8, NULL);
+            "width", G_TYPE_INT, wd,
+            "depth", G_TYPE_INT, ws, "signed", G_TYPE_BOOLEAN, wd != 8, NULL);
 
-        /* Add default MS channel layout if we have more than 2 channels,
-         * but the layout isn't specified like with WAVEEXT below. Not sure
-         * if this is right, but at least it makes sound output work at all
-         * in those cases. Somebody with a a 5.1 setup should double-check
-         * with chan-id.wav */
+        /* Add default channel layout. In theory this should be done
+         * for 1 and 2 channels too but apparently breaks too many
+         * things currently. Also we know no default layout for more than
+         * 8 channels. */
         if (ch > 2) {
-          guint32 channel_mask;
-
-          switch (ch) {
-            case 4:
-              channel_mask = 0x33;
-              break;
-            case 6:
-              channel_mask = 0x3f;
-              break;
-            default:
-              GST_WARNING ("don't know default layout for %d channels", ch);
-              channel_mask = 0;
-              break;
-          }
-
-          if (channel_mask) {
+          if (ch > 8)
+            GST_WARNING ("don't know default layout for %d channels", ch);
+          else if (gst_riff_wave_add_default_channel_layout (caps))
             GST_DEBUG ("using default channel layout for %d channels", ch);
-            if (!gst_riff_wavext_add_channel_layout (caps, channel_mask)) {
-              GST_WARNING ("failed to add channel layout");
-            }
-          }
+          else
+            GST_WARNING ("failed to add channel layout");
         }
       } else {
         /* FIXME: this is pretty useless - we need fixed caps */
         caps = gst_caps_from_string ("audio/x-raw-int, "
             "endianness = (int) LITTLE_ENDIAN, "
             "signed = (boolean) { true, false }, "
-            "width = (int) { 8, 16, 24, 32 }, "
-            "depth = (int) { 8, 16, 24, 32 }");
+            "width = (int) { 8, 16, 24, 32 }, " "depth = (int) [ 1, 32 ]");
       }
       if (codec_name && strf)
         *codec_name = g_strdup_printf ("Uncompressed %d-bit PCM audio",
@@ -812,6 +939,40 @@ gst_riff_create_audio_caps (guint16 codec_id,
       block_align = TRUE;
       break;
 
+    case GST_RIFF_WAVE_FORMAT_IEEE_FLOAT:
+      channels_max = 8;
+
+      if (strf != NULL) {
+        gint ba = strf->blockalign;
+        gint ch = strf->channels;
+        gint wd = ba * 8 / ch;
+
+        caps = gst_caps_new_simple ("audio/x-raw-float",
+            "endianness", G_TYPE_INT, G_LITTLE_ENDIAN,
+            "channels", G_TYPE_INT, ch, "width", G_TYPE_INT, wd, NULL);
+
+        /* Add default channel layout. In theory this should be done
+         * for 1 and 2 channels too but apparently breaks too many
+         * things currently. Also we know no default layout for more than
+         * 8 channels. */
+        if (ch > 2) {
+          if (ch > 8)
+            GST_WARNING ("don't know default layout for %d channels", ch);
+          else if (gst_riff_wave_add_default_channel_layout (caps))
+            GST_DEBUG ("using default channel layout for %d channels", ch);
+          else
+            GST_WARNING ("failed to add channel layout");
+        }
+      } else {
+        /* FIXME: this is pretty useless - we need fixed caps */
+        caps = gst_caps_from_string ("audio/x-raw-float, "
+            "endianness = (int) LITTLE_ENDIAN, " "width = (int) { 32, 64 }");
+      }
+      if (codec_name && strf)
+        *codec_name = g_strdup_printf ("Uncompressed %d-bit IEEE float audio",
+            strf->size);
+      break;
+
     case GST_RIFF_WAVE_FORMAT_IBM_CVSD:
       goto unknown;
 
@@ -821,16 +982,17 @@ gst_riff_create_audio_caps (guint16 codec_id,
           GST_WARNING ("invalid depth (%d) of alaw audio, overwriting.",
               strf->size);
           strf->size = 8;
-          strf->av_bps = 8;
-          strf->blockalign = strf->av_bps * strf->channels;
+          strf->blockalign = (strf->size * strf->channels) / 8;
+          strf->av_bps = strf->blockalign * strf->rate;
         }
         if (strf->av_bps == 0 || strf->blockalign == 0) {
           GST_WARNING ("fixing av_bps (%d) and blockalign (%d) of alaw audio",
               strf->av_bps, strf->blockalign);
-          strf->av_bps = strf->size;
-          strf->blockalign = strf->av_bps * strf->channels;
+          strf->blockalign = (strf->size * strf->channels) / 8;
+          strf->av_bps = strf->blockalign * strf->rate;
         }
       }
+      rate_max = 48000;
       caps = gst_caps_new_simple ("audio/x-alaw", NULL);
       if (codec_name)
         *codec_name = g_strdup ("A-law audio");
@@ -858,16 +1020,17 @@ gst_riff_create_audio_caps (guint16 codec_id,
           GST_WARNING ("invalid depth (%d) of mulaw audio, overwriting.",
               strf->size);
           strf->size = 8;
-          strf->av_bps = 8;
-          strf->blockalign = strf->av_bps * strf->channels;
+          strf->blockalign = (strf->size * strf->channels) / 8;
+          strf->av_bps = strf->blockalign * strf->rate;
         }
         if (strf->av_bps == 0 || strf->blockalign == 0) {
           GST_WARNING ("fixing av_bps (%d) and blockalign (%d) of mulaw audio",
               strf->av_bps, strf->blockalign);
-          strf->av_bps = strf->size;
-          strf->blockalign = strf->av_bps * strf->channels;
+          strf->blockalign = (strf->size * strf->channels) / 8;
+          strf->av_bps = strf->blockalign * strf->rate;
         }
       }
+      rate_max = 48000;
       caps = gst_caps_new_simple ("audio/x-mulaw", NULL);
       if (codec_name)
         *codec_name = g_strdup ("Mu-law audio");
@@ -877,6 +1040,7 @@ gst_riff_create_audio_caps (guint16 codec_id,
       goto unknown;
 
     case GST_RIFF_WAVE_FORMAT_DVI_ADPCM:
+      rate_max = 48000;
       caps = gst_caps_new_simple ("audio/x-adpcm",
           "layout", G_TYPE_STRING, "dvi", NULL);
       if (codec_name)
@@ -884,14 +1048,25 @@ gst_riff_create_audio_caps (guint16 codec_id,
       block_align = TRUE;
       break;
 
+    case GST_RIFF_WAVE_FORMAT_DSP_TRUESPEECH:
+      rate_min = 8000;
+      rate_max = 8000;
+      caps = gst_caps_new_simple ("audio/x-truespeech", NULL);
+      if (codec_name)
+        *codec_name = g_strdup ("DSP Group TrueSpeech");
+      break;
+
     case GST_RIFF_WAVE_FORMAT_GSM610:
     case GST_RIFF_WAVE_FORMAT_MSN:
+      rate_min = 1;
       caps = gst_caps_new_simple ("audio/ms-gsm", NULL);
       if (codec_name)
         *codec_name = g_strdup ("MS GSM audio");
       break;
 
     case GST_RIFF_WAVE_FORMAT_MPEGL12: /* mp1 or mp2 */
+      rate_min = 16000;
+      rate_max = 48000;
       caps = gst_caps_new_simple ("audio/mpeg",
           "mpegversion", G_TYPE_INT, 1, "layer", G_TYPE_INT, 2, NULL);
       if (codec_name)
@@ -899,6 +1074,8 @@ gst_riff_create_audio_caps (guint16 codec_id,
       break;
 
     case GST_RIFF_WAVE_FORMAT_MPEGL3:  /* mp3 */
+      rate_min = 8000;
+      rate_max = 48000;
       caps = gst_caps_new_simple ("audio/mpeg",
           "mpegversion", G_TYPE_INT, 1, "layer", G_TYPE_INT, 3, NULL);
       if (codec_name)
@@ -927,6 +1104,8 @@ gst_riff_create_audio_caps (guint16 codec_id,
       caps = gst_caps_new_simple ("audio/x-dts", NULL);
       if (codec_name)
         *codec_name = g_strdup ("DTS audio");
+      /* wavparse is not always able to specify rate/channels for DTS-in-wav */
+      rate_chan = FALSE;
       break;
     case GST_RIFF_WAVE_FORMAT_AAC:
     {
@@ -973,12 +1152,35 @@ gst_riff_create_audio_caps (guint16 codec_id,
         *codec_name = g_strdup ("Sony ATRAC3");
       break;
 
+    case GST_RIFF_WAVE_FORMAT_ADPCM_IMA_DK4:
+      rate_min = 8000;
+      rate_max = 96000;
+      channels_max = 2;
+      caps =
+          gst_caps_new_simple ("audio/x-adpcm", "layout", G_TYPE_STRING, "dk4",
+          NULL);
+      if (codec_name)
+        *codec_name = g_strdup ("IMA/DK4 ADPCM");
+      break;
+    case GST_RIFF_WAVE_FORMAT_ADPCM_IMA_DK3:
+      rate_min = 8000;
+      rate_max = 96000;
+      channels_max = 2;
+      caps =
+          gst_caps_new_simple ("audio/x-adpcm", "layout", G_TYPE_STRING, "dk3",
+          NULL);
+      if (codec_name)
+        *codec_name = g_strdup ("IMA/DK3 ADPCM");
+      break;
+
     case GST_RIFF_WAVE_FORMAT_EXTENSIBLE:{
       guint16 valid_bits_per_sample;
       guint32 channel_mask;
       guint32 subformat_guid[4];
       const guint8 *data;
 
+      channels_max = 8;
+
       /* should be at least 22 bytes */
       if (strf_data == NULL || GST_BUFFER_SIZE (strf_data) < 22) {
         GST_WARNING ("WAVE_FORMAT_EXTENSIBLE data size is %d (expected: 22)",
@@ -1006,21 +1208,37 @@ gst_riff_create_audio_caps (guint16 codec_id,
           GST_DEBUG ("PCM");
           if (strf != NULL) {
             gint ba = strf->blockalign;
-            gint ws = strf->size;
-            gint depth = ws;
+            gint wd = ba * 8 / strf->channels;
+            gint ws;
 
-            if (valid_bits_per_sample != 0)
-              depth = valid_bits_per_sample;
+            /* in riff, the depth is stored in the size field but it just
+             * means that the _least_ significant bits are cleared. We can
+             * therefore just play the sample as if it had a depth == width */
+            ws = wd;
+
+            /* For reference, use this to get the actual depth:
+             * ws = strf->size;
+             * if (valid_bits_per_sample != 0)
+             *   ws = valid_bits_per_sample; */
 
             caps = gst_caps_new_simple ("audio/x-raw-int",
                 "endianness", G_TYPE_INT, G_LITTLE_ENDIAN,
                 "channels", G_TYPE_INT, strf->channels,
-                "width", G_TYPE_INT, (int) (ba * 8 / strf->channels),
-                "depth", G_TYPE_INT, depth,
+                "width", G_TYPE_INT, wd,
+                "depth", G_TYPE_INT, ws,
                 "rate", G_TYPE_INT, strf->rate,
-                "signed", G_TYPE_BOOLEAN, (depth > 8) ? TRUE : FALSE, NULL);
+                "signed", G_TYPE_BOOLEAN, wd != 8, NULL);
 
-            if (!gst_riff_wavext_add_channel_layout (caps, channel_mask)) {
+            /* If channel_mask == 0 and channels > 2 let's
+             * assume default layout as some wav files don't have the
+             * channel mask set. Don't set the layout for 1 or 2
+             * channels as it apparently breaks too many things currently. */
+            if (channel_mask == 0 && strf->channels > 2)
+              channel_mask =
+                  gst_riff_wavext_get_default_channel_mask (strf->channels);
+
+            if ((channel_mask != 0 || strf->channels > 2) &&
+                !gst_riff_wavext_add_channel_layout (caps, channel_mask)) {
               GST_WARNING ("failed to add channel layout");
               gst_caps_unref (caps);
               caps = NULL;
@@ -1033,7 +1251,87 @@ gst_riff_create_audio_caps (guint16 codec_id,
             }
           }
         } else if (subformat_guid[0] == 0x00000003) {
-          GST_DEBUG ("FIXME: handle IEEE float format");
+          GST_DEBUG ("FLOAT");
+          if (strf != NULL) {
+            gint ba = strf->blockalign;
+            gint ws = strf->size;
+            gint wd = ba * 8 / strf->channels;
+
+            if (valid_bits_per_sample != 0)
+              ws = valid_bits_per_sample;
+
+            caps = gst_caps_new_simple ("audio/x-raw-float",
+                "endianness", G_TYPE_INT, G_LITTLE_ENDIAN,
+                "channels", G_TYPE_INT, strf->channels,
+                "width", G_TYPE_INT, wd, "rate", G_TYPE_INT, strf->rate, NULL);
+
+            /* If channel_mask == 0 and channels > 2 let's
+             * assume default layout as some wav files don't have the
+             * channel mask set. Don't set the layout for 1 or 2
+             * channels as it apparently breaks too many things currently. */
+            if (channel_mask == 0 && strf->channels > 2)
+              channel_mask =
+                  gst_riff_wavext_get_default_channel_mask (strf->channels);
+
+            if ((channel_mask != 0 || strf->channels > 2) &&
+                !gst_riff_wavext_add_channel_layout (caps, channel_mask)) {
+              GST_WARNING ("failed to add channel layout");
+              gst_caps_unref (caps);
+              caps = NULL;
+            }
+            rate_chan = FALSE;
+
+            if (codec_name) {
+              *codec_name =
+                  g_strdup_printf ("Uncompressed %d-bit IEEE float audio",
+                  strf->size);
+            }
+          }
+        } else if (subformat_guid[0] == 00000006) {
+          GST_DEBUG ("ALAW");
+          if (strf != NULL) {
+            if (strf->size != 8) {
+              GST_WARNING ("invalid depth (%d) of alaw audio, overwriting.",
+                  strf->size);
+              strf->size = 8;
+              strf->av_bps = 8;
+              strf->blockalign = strf->av_bps * strf->channels;
+            }
+            if (strf->av_bps == 0 || strf->blockalign == 0) {
+              GST_WARNING
+                  ("fixing av_bps (%d) and blockalign (%d) of alaw audio",
+                  strf->av_bps, strf->blockalign);
+              strf->av_bps = strf->size;
+              strf->blockalign = strf->av_bps * strf->channels;
+            }
+          }
+          rate_max = 48000;
+          caps = gst_caps_new_simple ("audio/x-alaw", NULL);
+
+          if (codec_name)
+            *codec_name = g_strdup ("A-law audio");
+        } else if (subformat_guid[0] == 0x00000007) {
+          GST_DEBUG ("MULAW");
+          if (strf != NULL) {
+            if (strf->size != 8) {
+              GST_WARNING ("invalid depth (%d) of mulaw audio, overwriting.",
+                  strf->size);
+              strf->size = 8;
+              strf->av_bps = 8;
+              strf->blockalign = strf->av_bps * strf->channels;
+            }
+            if (strf->av_bps == 0 || strf->blockalign == 0) {
+              GST_WARNING
+                  ("fixing av_bps (%d) and blockalign (%d) of mulaw audio",
+                  strf->av_bps, strf->blockalign);
+              strf->av_bps = strf->size;
+              strf->blockalign = strf->av_bps * strf->channels;
+            }
+          }
+          rate_max = 48000;
+          caps = gst_caps_new_simple ("audio/x-mulaw", NULL);
+          if (codec_name)
+            *codec_name = g_strdup ("Mu-law audio");
         } else if (subformat_guid[0] == 0x00000092) {
           GST_DEBUG ("FIXME: handle DOLBY AC3 SPDIF format");
         }
@@ -1052,7 +1350,6 @@ gst_riff_create_audio_caps (guint16 codec_id,
       break;
     }
       /* can anything decode these? pitfdll? */
-    case GST_RIFF_WAVE_FORMAT_VOXWARE:
     case GST_RIFF_WAVE_FORMAT_VOXWARE_BYTE_ALIGNED:
     case GST_RIFF_WAVE_FORMAT_VOXWARE_AC8:
     case GST_RIFF_WAVE_FORMAT_VOXWARE_AC10:
@@ -1079,6 +1376,11 @@ gst_riff_create_audio_caps (guint16 codec_id,
 
   if (strf != NULL) {
     if (rate_chan) {
+      if (strf->channels > channels_max)
+        goto too_many_channels;
+      if (strf->rate < rate_min || strf->rate > rate_max)
+        goto invalid_rate;
+
       gst_caps_set_simple (caps,
           "rate", G_TYPE_INT, strf->rate,
           "channels", G_TYPE_INT, strf->channels, NULL);
@@ -1106,6 +1408,20 @@ gst_riff_create_audio_caps (guint16 codec_id,
   }
 
   return caps;
+
+  /* ERROR */
+too_many_channels:
+  GST_WARNING
+      ("Stream claims to contain %u channels, but format only supports %d",
+      strf->channels, channels_max);
+  gst_caps_unref (caps);
+  return NULL;
+invalid_rate:
+  GST_WARNING
+      ("Stream with sample_rate %u, but format only supports %d .. %d",
+      strf->rate, rate_min, rate_max);
+  gst_caps_unref (caps);
+  return NULL;
 }
 
 GstCaps *
@@ -1152,6 +1468,7 @@ gst_riff_create_video_template_caps (void)
     GST_MAKE_FOURCC ('D', 'I', 'V', 'X'),
     GST_MAKE_FOURCC ('D', 'U', 'C', 'K'),
     GST_MAKE_FOURCC ('D', 'V', 'S', 'D'),
+    GST_MAKE_FOURCC ('D', 'V', '5', '0'),
     GST_MAKE_FOURCC ('D', 'X', '5', '0'),
     GST_MAKE_FOURCC ('F', 'L', 'V', '1'),
     GST_MAKE_FOURCC ('H', '2', '6', '3'),
@@ -1163,6 +1480,7 @@ gst_riff_create_video_template_caps (void)
     GST_MAKE_FOURCC ('I', 'V', '4', '1'),
     GST_MAKE_FOURCC ('I', 'V', '5', '0'),
     GST_MAKE_FOURCC ('L', '2', '6', '3'),
+    GST_MAKE_FOURCC ('L', '2', '6', '4'),
     GST_MAKE_FOURCC ('M', '2', '6', '3'),
     GST_MAKE_FOURCC ('M', '4', 'S', '2'),
     GST_MAKE_FOURCC ('M', 'J', 'P', 'G'),
@@ -1197,7 +1515,8 @@ gst_riff_create_video_template_caps (void)
     GST_MAKE_FOURCC ('c', 'v', 'i', 'd'),
     GST_MAKE_FOURCC ('h', '2', '6', '4'),
     GST_MAKE_FOURCC ('m', 's', 'v', 'c'),
-    GST_MAKE_FOURCC ('x', '2', '6', '3')
+    GST_MAKE_FOURCC ('x', '2', '6', '3'),
+    GST_MAKE_FOURCC ('d', 'r', 'a', 'c')
         /* FILL ME */
   };
   guint i;
@@ -1230,11 +1549,16 @@ gst_riff_create_audio_template_caps (void)
     GST_RIFF_WAVE_FORMAT_WMS,
     GST_RIFF_WAVE_FORMAT_ADPCM,
     GST_RIFF_WAVE_FORMAT_DVI_ADPCM,
+    GST_RIFF_WAVE_FORMAT_DSP_TRUESPEECH,
     GST_RIFF_WAVE_FORMAT_WMAV1,
     GST_RIFF_WAVE_FORMAT_WMAV2,
     GST_RIFF_WAVE_FORMAT_WMAV3,
-    GST_RIFF_WAVE_FORMAT_SONY_ATRAC3
-        /* FILL ME */
+    GST_RIFF_WAVE_FORMAT_SONY_ATRAC3,
+    GST_RIFF_WAVE_FORMAT_IEEE_FLOAT,
+    GST_RIFF_WAVE_FORMAT_VOXWARE_METASOUND,
+    GST_RIFF_WAVE_FORMAT_ADPCM_IMA_DK4,
+    GST_RIFF_WAVE_FORMAT_ADPCM_IMA_DK3,
+    /* FILL ME */
   };
   guint i;
   GstCaps *caps, *one;
