@@ -1,5 +1,5 @@
 /* GStreamer
- * Copyright (C) <2004> Wim Taymans <wim@fluendo.com>
+ * Copyright (C) <2004> Wim Taymans <wim.taymans@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -21,6 +21,8 @@
 #include "config.h"
 #endif
 
+#include <gst/gst-i18n-plugin.h>
+
 #include <string.h>
 #include <gst/gst.h>
 #include <gst/pbutils/pbutils.h>
@@ -52,6 +54,11 @@ GST_DEBUG_CATEGORY_STATIC (gst_decode_bin_debug);
 typedef struct _GstDecodeBin GstDecodeBin;
 typedef struct _GstDecodeBinClass GstDecodeBinClass;
 
+/**
+ * GstDecodeBin:
+ *
+ * Auto-plugging decoder element structure
+ */
 struct _GstDecodeBin
 {
   GstBin bin;                   /* we extend GstBin */
@@ -69,6 +76,7 @@ struct _GstDecodeBin
   gint numpads;
   gint numwaiting;
 
+  gboolean have_type;
   guint have_type_id;           /* signal id for the typefind element */
 
   gboolean shutting_down;       /* stop pluggin if we're shutting down */
@@ -152,6 +160,8 @@ static void new_caps (GstPad * pad, GParamSpec * unused, GstDynamic * dynamic);
 static void queue_filled_cb (GstElement * queue, GstDecodeBin * decode_bin);
 static void queue_underrun_cb (GstElement * queue, GstDecodeBin * decode_bin);
 
+static gboolean is_demuxer_element (GstElement * srcelement);
+
 static GstElementClass *parent_class;
 static guint gst_decode_bin_signals[LAST_SIGNAL] = { 0 };
 
@@ -159,7 +169,7 @@ static const GstElementDetails gst_decode_bin_details =
 GST_ELEMENT_DETAILS ("Decoder Bin",
     "Generic/Bin/Decoder",
     "Autoplug and decode to raw media",
-    "Wim Taymans <wim@fluendo.com>");
+    "Wim Taymans <wim.taymans@gmail.com>");
 
 
 static GType
@@ -202,21 +212,47 @@ gst_decode_bin_class_init (GstDecodeBinClass * klass)
 
   parent_class = g_type_class_peek_parent (klass);
 
+  /**
+   * GstDecodeBin::new-decoded-pad:
+   * @bin: The decodebin
+   * @pad: The newly created pad
+   * @islast: #TRUE if this is the last pad to be added. Deprecated.
+   *
+   * This signal gets emitted as soon as a new pad of the same type as one of
+   * the valid 'raw' types is added.
+   */
   gst_decode_bin_signals[SIGNAL_NEW_DECODED_PAD] =
       g_signal_new ("new-decoded-pad", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_LAST,
       G_STRUCT_OFFSET (GstDecodeBinClass, new_decoded_pad), NULL, NULL,
       gst_play_marshal_VOID__OBJECT_BOOLEAN, G_TYPE_NONE, 2, GST_TYPE_PAD,
       G_TYPE_BOOLEAN);
+  /**
+   * GstDecodeBin::removed-decoded-pad:
+   * @bin: The decodebin
+   * @pad: The pad that was removed
+   *
+   * This signal is emitted when a 'final' caps pad has been removed.
+   */
   gst_decode_bin_signals[SIGNAL_REMOVED_DECODED_PAD] =
       g_signal_new ("removed-decoded-pad", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_LAST,
       G_STRUCT_OFFSET (GstDecodeBinClass, removed_decoded_pad), NULL, NULL,
       gst_marshal_VOID__OBJECT, G_TYPE_NONE, 1, GST_TYPE_PAD);
+  /**
+   * GstDecodeBin::unknown-type:
+   * @bin: The decodebin
+   * @pad: The new pad containing caps that cannot be resolved to a 'final'
+   *       stream type.
+   * @caps: The #GstCaps of the pad that cannot be resolved.
+   *
+   * This signal is emitted when a pad for which there is no further possible
+   * decoding is added to the decodebin.
+   */
   gst_decode_bin_signals[SIGNAL_UNKNOWN_TYPE] =
       g_signal_new ("unknown-type", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (GstDecodeBinClass, unknown_type),
-      NULL, NULL, gst_marshal_VOID__OBJECT_OBJECT, G_TYPE_NONE, 2,
+      NULL, NULL, gst_marshal_VOID__OBJECT_BOXED, G_TYPE_NONE, 2,
       GST_TYPE_PAD, GST_TYPE_CAPS);
 
   gobject_klass->dispose = GST_DEBUG_FUNCPTR (gst_decode_bin_dispose);
@@ -235,7 +271,7 @@ gst_decode_bin_class_init (GstDecodeBinClass * klass)
 
 /* check if the bin is dynamic.
  *
- * If there are no outstanding dynamic connections, the bin is 
+ * If there are no outstanding dynamic connections, the bin is
  * considered to be non-dynamic.
  */
 static gboolean
@@ -334,7 +370,7 @@ gst_decode_bin_init (GstDecodeBin * decode_bin)
     }
 
     /* get the sinkpad */
-    pad = gst_element_get_pad (decode_bin->typefind, "sink");
+    pad = gst_element_get_static_pad (decode_bin->typefind, "sink");
 
     /* ghost the sink pad to ourself */
     gpad = gst_ghost_pad_new ("sink", pad);
@@ -494,7 +530,7 @@ free_dynamics (GstDecodeBin * decode_bin)
 }
 
 /* this function runs through the element factories and returns a list
- * of all elements that are able to sink the given caps 
+ * of all elements that are able to sink the given caps
  */
 static GList *
 find_compatibles (GstDecodeBin * decode_bin, const GstCaps * caps)
@@ -566,6 +602,25 @@ free_pad_probes (GstDecodeBin * decode_bin)
   decode_bin->probes = NULL;
 }
 
+/* used when we need to remove a probe because the decoder we plugged failed
+ * to activate */
+static void
+free_pad_probe_for_element (GstDecodeBin * decode_bin, GstElement * element)
+{
+  GList *l;
+
+  for (l = decode_bin->probes; l != NULL; l = g_list_next (l)) {
+    PadProbeData *data = (PadProbeData *) l->data;
+
+    if (GST_ELEMENT_CAST (GST_PAD_PARENT (data->pad)) == element) {
+      gst_pad_remove_data_probe (data->pad, data->sigid);
+      decode_bin->probes = g_list_delete_link (decode_bin->probes, l);
+      g_free (data);
+      return;
+    }
+  }
+}
+
 static gboolean
 add_fakesink (GstDecodeBin * decode_bin)
 {
@@ -581,6 +636,7 @@ add_fakesink (GstDecodeBin * decode_bin)
   /* hacky, remove sink flag, we don't want our decodebin to become a sink
    * just because we add a fakesink element to make us ASYNC */
   GST_OBJECT_FLAG_UNSET (decode_bin->fakesink, GST_ELEMENT_IS_SINK);
+
   /* takes ownership */
   if (!gst_bin_add (GST_BIN (decode_bin), decode_bin->fakesink)) {
     g_warning ("Could not add fakesink element, decodebin will not work");
@@ -611,6 +667,9 @@ remove_fakesink (GstDecodeBin * decode_bin)
   if (decode_bin->fakesink) {
     GST_DEBUG_OBJECT (decode_bin, "Removing fakesink and marking state dirty");
 
+    /* Lock the state to prevent it from changing state to non-NULL
+     * before it's removed */
+    gst_element_set_locked_state (decode_bin->fakesink, TRUE);
     /* setting the state to NULL is never async */
     gst_element_set_state (decode_bin->fakesink, GST_STATE_NULL);
     gst_bin_remove (GST_BIN (decode_bin), decode_bin->fakesink);
@@ -622,9 +681,6 @@ remove_fakesink (GstDecodeBin * decode_bin)
 
   if (removed_fakesink) {
     free_pad_probes (decode_bin);
-
-    gst_element_post_message (GST_ELEMENT_CAST (decode_bin),
-        gst_message_new_state_dirty (GST_OBJECT_CAST (decode_bin)));
   }
 }
 
@@ -665,13 +721,54 @@ pad_probe (GstPad * pad, GstMiniObject * data, GstDecodeBin * decode_bin)
   return TRUE;
 }
 
+/* FIXME: this should be somehow merged with the queue code in
+ * try_to_link_1() to reduce code duplication */
+static GstPad *
+add_raw_queue (GstDecodeBin * decode_bin, GstPad * pad)
+{
+  GstElement *queue = NULL;
+  GstPad *queuesinkpad = NULL, *queuesrcpad = NULL;
+
+  queue = gst_element_factory_make ("queue", NULL);
+  decode_bin->queue_type = G_OBJECT_TYPE (queue);
+
+  g_object_set (G_OBJECT (queue), "max-size-buffers", 0, NULL);
+  g_object_set (G_OBJECT (queue), "max-size-time", G_GINT64_CONSTANT (0), NULL);
+  g_object_set (G_OBJECT (queue), "max-size-bytes", 8192, NULL);
+  gst_bin_add (GST_BIN (decode_bin), queue);
+  gst_element_set_state (queue, GST_STATE_READY);
+  queuesinkpad = gst_element_get_static_pad (queue, "sink");
+  queuesrcpad = gst_element_get_static_pad (queue, "src");
+
+  if (gst_pad_link (pad, queuesinkpad) != GST_PAD_LINK_OK) {
+    GST_WARNING_OBJECT (decode_bin,
+        "Linking queue failed, trying without queue");
+    gst_element_set_state (queue, GST_STATE_NULL);
+    gst_object_unref (queuesrcpad);
+    gst_object_unref (queuesinkpad);
+    gst_bin_remove (GST_BIN (decode_bin), queue);
+    return gst_object_ref (pad);
+  }
+
+  decode_bin->queues = g_list_append (decode_bin->queues, queue);
+  g_signal_connect (G_OBJECT (queue),
+      "overrun", G_CALLBACK (queue_filled_cb), decode_bin);
+  g_signal_connect (G_OBJECT (queue),
+      "underrun", G_CALLBACK (queue_underrun_cb), decode_bin);
+
+  gst_element_set_state (queue, GST_STATE_PAUSED);
+  gst_object_unref (queuesinkpad);
+
+  return queuesrcpad;
+}
+
 /* given a pad and a caps from an element, find the list of elements
  * that could connect to the pad
  *
  * If the pad has a raw format, this function will create a ghostpad
  * for the pad onto the decodebin.
  *
- * If no compatible elements could be found, this function will signal 
+ * If no compatible elements could be found, this function will signal
  * the unknown_type signal.
  */
 static void
@@ -717,6 +814,17 @@ close_pad_link (GstElement * element, GstPad * pad, GstCaps * caps,
     GstPad *ghost;
     PadProbeData *data;
 
+    /* If we're at a demuxer element but have raw data already
+     * we have to add a queue here. For non-raw data this is done
+     * in try_to_link_1() */
+    if (is_demuxer_element (element)) {
+      GST_DEBUG_OBJECT (decode_bin,
+          "Element %s is a demuxer, inserting a queue",
+          GST_OBJECT_NAME (element));
+
+      pad = add_raw_queue (decode_bin, pad);
+    }
+
     /* make a unique name for this new pad */
     padname = g_strdup_printf ("src%d", decode_bin->numpads);
     decode_bin->numpads++;
@@ -746,11 +854,16 @@ close_pad_link (GstElement * element, GstPad * pad, GstCaps * caps,
     GST_DEBUG_OBJECT (decode_bin, "emitted new-decoded-pad");
 
     g_free (padname);
+
+    /* If we're at a demuxer element pad was set to a queue's
+     * srcpad and must be unref'd here */
+    if (is_demuxer_element (element))
+      gst_object_unref (pad);
   } else {
     GList *to_try;
 
     /* if the caps has many types, we need to delay */
-    if (gst_caps_get_size (caps) != 1)
+    if (!gst_caps_is_fixed (caps))
       goto many_types;
 
     /* continue plugging, first find all compatible elements */
@@ -779,6 +892,19 @@ unknown_type:
 
     gst_element_post_message (GST_ELEMENT_CAST (decode_bin),
         gst_missing_decoder_message_new (GST_ELEMENT_CAST (decode_bin), caps));
+
+    if (element == decode_bin->typefind) {
+      gchar *desc;
+
+      desc = gst_pb_utils_get_decoder_description (caps);
+      GST_ELEMENT_ERROR (decode_bin, STREAM, CODEC_NOT_FOUND,
+          (_("A %s plugin is required to play this stream, but not installed."),
+              desc),
+          ("No decoder to handle media type '%s'",
+              gst_structure_get_name (gst_caps_get_structure (caps, 0))));
+      g_free (desc);
+    }
+
     return;
   }
 dont_know_yet:
@@ -799,7 +925,7 @@ setup_caps_delay:
   }
 }
 
-/* Decide whether an element is a demuxer based on the 
+/* Decide whether an element is a demuxer based on the
  * klass and number/type of src pad templates it has */
 static gboolean
 is_demuxer_element (GstElement * srcelement)
@@ -886,8 +1012,8 @@ try_to_link_1 (GstDecodeBin * decode_bin, GstElement * srcelement, GstPad * pad,
     g_object_set (G_OBJECT (queue), "max-size-bytes", 8192, NULL);
     gst_bin_add (GST_BIN (decode_bin), queue);
     gst_element_set_state (queue, GST_STATE_READY);
-    queuesinkpad = gst_element_get_pad (queue, "sink");
-    usedsrcpad = queuesrcpad = gst_element_get_pad (queue, "src");
+    queuesinkpad = gst_element_get_static_pad (queue, "sink");
+    usedsrcpad = queuesrcpad = gst_element_get_static_pad (queue, "src");
 
     dqlink = gst_pad_link (pad, queuesinkpad);
     g_return_val_if_fail (dqlink == GST_PAD_LINK_OK, NULL);
@@ -912,9 +1038,9 @@ try_to_link_1 (GstDecodeBin * decode_bin, GstElement * srcelement, GstPad * pad,
     }
 
     /* try to link the given pad to a sinkpad */
-    /* FIXME, find the sinkpad by looping over the pads instead of 
+    /* FIXME, find the sinkpad by looping over the pads instead of
      * looking it up by name */
-    if ((sinkpad = gst_element_get_pad (element, "sink")) == NULL) {
+    if ((sinkpad = gst_element_get_static_pad (element, "sink")) == NULL) {
       /* if no pad is found we can't do anything */
       GST_WARNING_OBJECT (decode_bin, "could not find sinkpad in element");
       continue;
@@ -925,7 +1051,19 @@ try_to_link_1 (GstDecodeBin * decode_bin, GstElement * srcelement, GstPad * pad,
     gst_bin_add (GST_BIN (decode_bin), element);
 
     /* set to READY first so it is ready, duh. */
-    gst_element_set_state (element, GST_STATE_READY);
+    if (gst_element_set_state (element,
+            GST_STATE_READY) == GST_STATE_CHANGE_FAILURE) {
+      GST_WARNING_OBJECT (decode_bin, "Couldn't set %s to READY",
+          GST_ELEMENT_NAME (element));
+      /* get rid of the sinkpad */
+      gst_object_unref (sinkpad);
+      /* this element did not work, remove it again and continue trying
+       * other elements, the element will be disposed. */
+      /* FIXME: shouldn't we do this before adding it to the bin so that no
+       * error messages get through to the app? (tpm) */
+      gst_bin_remove (GST_BIN (decode_bin), element);
+      continue;
+    }
 
     if ((ret = gst_pad_link (usedsrcpad, sinkpad)) != GST_PAD_LINK_OK) {
       GST_DEBUG_OBJECT (decode_bin, "link failed on pad %s:%s, reason %d",
@@ -960,15 +1098,25 @@ try_to_link_1 (GstDecodeBin * decode_bin, GstElement * srcelement, GstPad * pad,
       /* now that we added the element we can try to continue autoplugging
        * on it until we have a raw type */
       close_link (element, decode_bin);
+
       /* change the state of the element to that of the parent */
-      gst_element_set_state (element, GST_STATE_PAUSED);
+      if ((gst_element_set_state (element,
+                  GST_STATE_PAUSED)) == GST_STATE_CHANGE_FAILURE) {
+        GST_WARNING_OBJECT (decode_bin, "Couldn't set %s to PAUSED",
+            GST_ELEMENT_NAME (element));
+        /* close_link -> close_pad_link -> might have set up a pad probe */
+        free_pad_probe_for_element (decode_bin, element);
+        gst_element_set_state (element, GST_STATE_NULL);
+        gst_bin_remove (GST_BIN (decode_bin), element);
+        continue;
+      }
 
       result = element;
 
       /* get rid of the sinkpad now */
       gst_object_unref (sinkpad);
 
-      /* Set the queue to paused and set the pointer to NULL so we don't 
+      /* Set the queue to paused and set the pointer to NULL so we don't
        * remove it below */
       if (queue != NULL) {
         gst_element_set_state (queue, GST_STATE_PAUSED);
@@ -1048,7 +1196,7 @@ get_our_ghost_pad (GstDecodeBin * decode_bin, GstPad * pad)
 }
 
 /* remove all downstream elements starting from the given pad.
- * Also make sure to remove the ghostpad we created for the raw 
+ * Also make sure to remove the ghostpad we created for the raw
  * decoded stream.
  */
 static void
@@ -1070,7 +1218,7 @@ remove_element_chain (GstDecodeBin * decode_bin, GstPad * pad)
   GST_DEBUG_OBJECT (decode_bin, "%s:%s", GST_DEBUG_PAD_NAME (pad));
   int_links = gst_pad_get_internal_links (pad);
 
-  /* remove all elements linked to this pad up to the ghostpad 
+  /* remove all elements linked to this pad up to the ghostpad
    * that we created for this stream */
   for (walk = int_links; walk; walk = g_list_next (walk)) {
     GstPad *pad;
@@ -1134,7 +1282,7 @@ remove_element_chain (GstDecodeBin * decode_bin, GstPad * pad)
   gst_bin_remove (GST_BIN (decode_bin), elem);
 }
 
-/* there are @bytes bytes in @queue, enlarge it 
+/* there are @bytes bytes in @queue, enlarge it
  *
  * Returns: new max number of bytes in @queue
  */
@@ -1167,13 +1315,14 @@ queue_underrun_cb (GstElement * queue, GstDecodeBin * decode_bin)
 {
   /* FIXME: we don't really do anything here for now. Ideally we should
    * see if some of the queues are filled and increase their values
-   * in that case. 
+   * in that case.
    * Note: be very carefull with thread safety here as this underrun
    * signal is done from the streaming thread of queue srcpad which
    * is different from the pad_added (where we add the queue to the
    * list) and the overrun signals that are signalled from the
    * demuxer thread.
    */
+  GST_DEBUG_OBJECT (decode_bin, "got underrun");
 }
 
 /* Make sure we don't have a full queue and empty queue situation */
@@ -1259,11 +1408,13 @@ new_pad (GstElement * element, GstPad * pad, GstDynamic * dynamic)
 
 shutting_down1:
   {
+    GST_DEBUG_OBJECT (decode_bin, "we are shutting down");
     GST_OBJECT_UNLOCK (decode_bin);
     return;
   }
 shutting_down2:
   {
+    GST_DEBUG_OBJECT (decode_bin, "we are shutting down");
     GST_STATE_UNLOCK (decode_bin);
     return;
   }
@@ -1360,7 +1511,7 @@ elem_is_dynamic (GstElement * element, GstDecodeBin * decode_bin)
       {
         /* try to get the pad to see if it is already created or
          * not */
-        GstPad *pad = gst_element_get_pad (element, templ_name);
+        GstPad *pad = gst_element_get_static_pad (element, templ_name);
 
         if (pad) {
           GST_DEBUG_OBJECT (decode_bin, "got the pad for sometimes template %s",
@@ -1447,7 +1598,7 @@ close_link (GstElement * element, GstDecodeBin * decode_bin)
       case GST_PAD_ALWAYS:
       {
         /* get the pad that we need to autoplug */
-        GstPad *pad = gst_element_get_pad (element, templ_name);
+        GstPad *pad = gst_element_get_static_pad (element, templ_name);
 
         if (pad) {
           GST_DEBUG_OBJECT (decode_bin, "got the pad for always template %s",
@@ -1466,7 +1617,7 @@ close_link (GstElement * element, GstDecodeBin * decode_bin)
       {
         /* try to get the pad to see if it is already created or
          * not */
-        GstPad *pad = gst_element_get_pad (element, templ_name);
+        GstPad *pad = gst_element_get_static_pad (element, templ_name);
 
         if (pad) {
           GST_DEBUG_OBJECT (decode_bin, "got the pad for sometimes template %s",
@@ -1497,7 +1648,7 @@ close_link (GstElement * element, GstDecodeBin * decode_bin)
   }
 
   /* Check if this is an element with more than 1 pad. If this element
-   * has more than 1 pad, we need to be carefull not to signal the 
+   * has more than 1 pad, we need to be carefull not to signal the
    * no_more_pads signal after connecting the first pad. */
   more = g_list_length (to_connect) > 1;
 
@@ -1535,26 +1686,39 @@ type_found (GstElement * typefind, guint probability, GstCaps * caps,
   gboolean dynamic;
   GstPad *pad;
 
-  GST_STATE_LOCK (decode_bin);
-  if (decode_bin->shutting_down)
-    goto shutting_down;
-
   GST_DEBUG_OBJECT (decode_bin, "typefind found caps %" GST_PTR_FORMAT, caps);
 
+  GST_OBJECT_LOCK (decode_bin);
+  if (decode_bin->shutting_down)
+    goto shutting_down;
+  GST_OBJECT_UNLOCK (decode_bin);
+
+  GST_STATE_LOCK (decode_bin);
+  if (decode_bin->shutting_down)
+    goto exit;
+
+  /* don't need the typefind anymore if we already found a type, we're not going
+   * to be able to do anything with it anyway except for generating errors */
+  if (decode_bin->have_type)
+    goto exit;
+
+  decode_bin->have_type = TRUE;
+
   /* special-case text/plain: we only want to accept it as a raw type if it
-   * comes from a subtitel parser element or a demuxer, but not if it is the
+   * comes from a subtitle parser element or a demuxer, but not if it is the
    * type of the entire stream, in which case we just want to error out */
   if (typefind == decode_bin->typefind &&
       gst_structure_has_name (gst_caps_get_structure (caps, 0), "text/plain")) {
     gst_element_no_more_pads (GST_ELEMENT (decode_bin));
     /* we can't handle this type of stream */
-    GST_ELEMENT_ERROR (decode_bin, STREAM, WRONG_TYPE, (NULL),
+    GST_ELEMENT_ERROR (decode_bin, STREAM, WRONG_TYPE,
+        (_("This appears to be a text file")),
         ("decodebin cannot decode plain text files"));
-    goto shutting_down;
+    goto exit;
   }
 
   /* autoplug the new pad with the caps that the signal gave us. */
-  pad = gst_element_get_pad (typefind, "src");
+  pad = gst_element_get_static_pad (typefind, "src");
   close_pad_link (typefind, pad, caps, decode_bin, FALSE);
   gst_object_unref (pad);
 
@@ -1569,11 +1733,46 @@ type_found (GstElement * typefind, guint probability, GstCaps * caps,
     GST_DEBUG_OBJECT (decode_bin, "we have more dynamic elements");
   }
 
-shutting_down:
+exit:
   GST_STATE_UNLOCK (decode_bin);
   return;
+
+shutting_down:
+  {
+    GST_DEBUG_OBJECT (decode_bin, "we are shutting down");
+    GST_OBJECT_UNLOCK (decode_bin);
+    return;
+  }
 }
 
+static void
+disconnect_unlinked_signals (GstDecodeBin * decode_bin, GstElement * element)
+{
+  GstIterator *pad_it = NULL;
+  gboolean done = FALSE;
+
+  pad_it = gst_element_iterate_src_pads (element);
+  while (!done) {
+    GstPad *pad = NULL;
+
+    switch (gst_iterator_next (pad_it, (gpointer) & pad)) {
+      case GST_ITERATOR_OK:
+        g_signal_handlers_disconnect_by_func (pad, (gpointer) unlinked,
+            decode_bin);
+        gst_object_unref (pad);
+        break;
+      case GST_ITERATOR_RESYNC:
+        gst_iterator_resync (pad_it);
+        break;
+      default:
+        done = TRUE;
+        break;
+    }
+  }
+  gst_iterator_free (pad_it);
+}
+
+
 static void
 cleanup_decodebin (GstDecodeBin * decode_bin)
 {
@@ -1585,7 +1784,7 @@ cleanup_decodebin (GstDecodeBin * decode_bin)
 
   GST_DEBUG_OBJECT (decode_bin, "cleaning up decodebin");
 
-  typefind_pad = gst_element_get_pad (decode_bin->typefind, "src");
+  typefind_pad = gst_element_get_static_pad (decode_bin->typefind, "src");
   if (GST_IS_PAD (typefind_pad)) {
     g_signal_handlers_block_by_func (typefind_pad, (gpointer) unlinked,
         decode_bin);
@@ -1599,8 +1798,7 @@ cleanup_decodebin (GstDecodeBin * decode_bin)
       case GST_ITERATOR_OK:
         if (element != decode_bin->typefind && element != decode_bin->fakesink) {
           GST_DEBUG_OBJECT (element, "removing autoplugged element");
-          g_signal_handlers_disconnect_by_func (element, (gpointer) unlinked,
-              decode_bin);
+          disconnect_unlinked_signals (decode_bin, element);
           gst_element_set_state (element, GST_STATE_NULL);
           gst_bin_remove (GST_BIN (decode_bin), element);
         }
@@ -1650,6 +1848,8 @@ cleanup_decodebin (GstDecodeBin * decode_bin)
   if (GST_IS_PAD (typefind_pad)) {
     g_signal_handlers_unblock_by_func (typefind_pad, (gpointer) unlinked,
         decode_bin);
+    g_signal_handlers_disconnect_by_func (typefind_pad, (gpointer) unlinked,
+        decode_bin);
     gst_object_unref (typefind_pad);
   }
 }
@@ -1673,6 +1873,7 @@ gst_decode_bin_change_state (GstElement * element, GstStateChange transition)
     case GST_STATE_CHANGE_READY_TO_PAUSED:
       GST_OBJECT_LOCK (decode_bin);
       decode_bin->shutting_down = FALSE;
+      decode_bin->have_type = FALSE;
       GST_OBJECT_UNLOCK (decode_bin);
 
       if (!add_fakesink (decode_bin))
@@ -1730,6 +1931,13 @@ plugin_init (GstPlugin * plugin)
 {
   GST_DEBUG_CATEGORY_INIT (gst_decode_bin_debug, "decodebin", 0, "decoder bin");
 
+#ifdef ENABLE_NLS
+  GST_DEBUG ("binding text domain %s to locale dir %s", GETTEXT_PACKAGE,
+      LOCALEDIR);
+  bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
+  bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+#endif /* ENABLE_NLS */
+
   return gst_element_register (plugin, "decodebin", GST_RANK_NONE,
       GST_TYPE_DECODE_BIN);
 }
