@@ -263,14 +263,14 @@ test_pipeline (const char *pipeline)
   bin = gst_parse_launch (pipeline, &error);
   fail_unless (bin != NULL, "Error parsing pipeline: %s",
       error ? error->message : "(invalid error)");
-  pad = gst_bin_find_unconnected_pad (GST_BIN (bin), GST_PAD_SRC);
+  pad = gst_bin_find_unlinked_pad (GST_BIN (bin), GST_PAD_SRC);
   fail_unless (pad != NULL, "Could not locate free src pad");
 
   /* connect the fake sink */
   sink = gst_element_factory_make ("fakesink", "fake_sink");
   fail_unless (sink != NULL, "Could create fakesink");
   fail_unless (gst_bin_add (GST_BIN (bin), sink), "Could not insert fakesink");
-  sinkpad = gst_element_get_pad (sink, "sink");
+  sinkpad = gst_element_get_static_pad (sink, "sink");
   fail_unless (sinkpad != NULL, "Could not get fakesink src pad");
 
   linkret = gst_pad_link (pad, sinkpad);
@@ -294,6 +294,7 @@ test_pipeline (const char *pipeline)
   gst_object_unref (bin);
 }
 
+#ifdef HAVE_VORBIS
 GST_START_TEST (test_vorbis)
 {
   test_pipeline
@@ -301,7 +302,9 @@ GST_START_TEST (test_vorbis)
 }
 
 GST_END_TEST;
+#endif
 
+#ifdef HAVE_THEORA
 GST_START_TEST (test_theora)
 {
   test_pipeline
@@ -309,7 +312,9 @@ GST_START_TEST (test_theora)
 }
 
 GST_END_TEST;
+#endif
 
+#if (defined (HAVE_THEORA) && defined (HAVE_VORBIS))
 GST_START_TEST (test_theora_vorbis)
 {
   test_pipeline
@@ -327,6 +332,7 @@ GST_START_TEST (test_vorbis_theora)
 }
 
 GST_END_TEST;
+#endif
 
 GST_START_TEST (test_simple_cleanup)
 {
